#include <stdio.h>
#include "neo.h"

void neo_boardport_create_null(neo_boardport *bp)
{
  bp->starboard=0;
  bp->board=0;
  bp->starport=0;
  bp->port=0;
  bp->hasboard=0;
  bp->hasport=0;
}

void neo_boardport_create_from_port_int(neo_boardport *bp, int portnum)
{
  bp->starboard=0;
  bp->board=0;
  bp->starport=0;
  bp->port=portnum;
  bp->hasboard=0;
  bp->hasport=1;
}

void neo_boardport_create_from_bp_ints(neo_boardport *bp, int boardnum, int portnum)
{
  bp->starboard=0;
  bp->board=boardnum;
  bp->starport=0;
  bp->port=portnum;
  bp->hasboard=1;
  bp->hasport=1;
}

void neo_boardport_create_from_string(neo_boardport *bp, char *in)
{
  char *ptr;
  char board[1024], port[1024];

  bp->starboard=0;
  bp->board=0;
  bp->starport=0;
  bp->port=0;
  bp->hasboard=0;
  bp->hasport=0;

  ptr=strchr(in, '/');
  if (ptr==NULL) {
    /* it's only a port */
    bp->hasport=1;
    bp->hasboard=0;
    strcpy(port, in);
    if (!strcmp(port, "*") || !strcmp(port, ".")) {
      bp->starport=1;
    } else {
      bp->port=atoi(port);
    }
  } else {
    /* first get the board */
    strncpy(board, in, ptr-in);
    board[ptr-in]='\0';
    bp->hasboard=1;
    if (!strcmp(board, "*") || !strcmp(board, ".")) {
      bp->starboard=1;
    } else {
      bp->board=atoi(board);
    }

    /* now get the port if there is one */
    if (ptr+1 == '\0') {
      /* no port */
      bp->hasport=0;
    } else {
      /* there is a port */
      bp->hasport=1;
      strcpy(port, ptr+1);
      if (!strcmp(port, "*") || !strcmp(port, ".")) {
	bp->starport=1;
      } else {
	bp->port=atoi(port);
      }
    }
  }
}

int neo_boardport_has_port(neo_boardport *bp)
{
  return(bp->hasport);
}

int neo_boardport_has_board(neo_boardport *bp)
{
  return(bp->hasboard);
}

int neo_boardport_star_port(neo_boardport *bp)
{
  return(bp->starport);
}

int neo_boardport_star_board(neo_boardport *bp)
{
  return(bp->starboard);
}

int neo_boardport_get_portnum(neo_boardport *bp)
{
  if (bp->hasport) return(bp->port);
  return(-1);
}

int neo_boardport_get_boardnum(neo_boardport *bp)
{
  if (bp->hasboard) return(bp->board);
  return(-1);
}

/*
 * caller must free the return
 */
char *neo_boardport_to_string(neo_boardport *bp)
{
  char *out;
  out=malloc(10);
  strcpy(out, "");
  if (bp->hasboard) {
    if (!bp->starboard) {
      sprintf(out, "%i/", bp->board);
    } else {
      sprintf(out, "*/");
    }
  }

  if (bp->hasport) {
    if (!bp->starport) {
      sprintf(out, "%s%i", out, bp->port);
    } else {
      sprintf(out, "%s*", out);
    }
  }
  return(out);
}

int neo_boardport_is_equal(neo_boardport *a, neo_boardport *b)
{
  if (a->hasboard != b->hasboard) return(0);
  if (a->board != b->board) return(0);
  if (a->starboard != b->starboard) return(0);
  if (a->hasport != b->hasport) return(0);
  if (a->port != b->port) return(0);
  if (a->starport != b-> starport) return(0);
  return(1);
}

void neo_boardport_copy(neo_boardport *to, neo_boardport *from)
{
  to->hasboard=from->hasboard;
  to->board=from->board;
  to->starboard=from->starboard;
  to->hasport=from->hasport;
  to->port=from->port;
  to->starport=from->starport;
}

void neo_boardport_free(neo_boardport *bp)
{
  /* Intentionally nothing to do given the current
   * implementation
   */
}
