#include "neo.h"
#include <stdlib.h>

#define INITSIZE 30
#define GROWAT 2
#define GROWBY 1.5

int neo_list_create(neo_list *l)
{
  l->size=0;
  l->list=(void **)malloc(INITSIZE*sizeof(void *));
  l->avail=INITSIZE;
  if (l->list==NULL) return(-1);
  return(0);
}

int neo_list_get_size(neo_list *l)
{
  return(l->size);
}

void *neo_list_get_element(neo_list *l, int n)
{
  if (n>l->size-1) return(NULL);
  return(l->list[n]);
}

int neo_list_append_element(neo_list *l, void *element)
{
  void *ptr;
  
  if ((l->size+1) > (l->avail/GROWAT)) {
    ptr=realloc(l->list, l->avail*GROWBY*sizeof(void *));
    if (ptr==NULL) return(-1);
    l->list=ptr;
    l->avail=l->avail*GROWBY;
  }

  l->list[l->size]=element;
  l->size++;
  return(0);
}

int neo_list_prepend_element(neo_list *l, void *element)
{
  void *ptr;
  int i;
 
  if ((l->size+1) > (l->avail/GROWAT)) {
    ptr=realloc(l->list, l->avail*GROWBY*sizeof(void *));
    if (ptr==NULL) return(-1);
    l->list=ptr;
    l->avail=l->avail*GROWBY;
  }

  for (i=l->size; i>0; i--) {
    l->list[i]=l->list[i-1];
  }
  l->list[0]=element;
  l->size++;
  return(0);
}

int neo_list_remove_element(neo_list *l, int n)
{
  int i;

  if (n>l->size-1) return(-1);
  for (i=n; i<l->size-1; i++) {
    l->list[i]=l->list[i+1];
  }
  l->size--;
  return(0);
}

/* todo: might leak memory */
int neo_list_replace_element(neo_list *l, int n, void *element)
{
  if (n>l->size-1) return(-1);

  l->list[n]=element;
  return(0);
}

void neo_list_copy(neo_list *src, neo_list *dst)
{
  dst->size=src->size;
  dst->avail=src->avail;
  dst->list=src->list;
}

/* 
 *  Free the list and each element as well
 */
void neo_list_free_all(neo_list *l, void (*elefree)(void *))
{
  int i;

  for (i=0; i<l->size; i++) {
    (elefree)(l->list[i]);
  }
  free(l->list);
}

/*
 * Just free the list, leave the elements alone
 */
void neo_list_free_simple(neo_list *l)
{
  if (l->list) free(l->list);
}
